@@ -11,6 +11,11 @@
 (function() {
 'use strict';
 
+if (!Element.prototype.matches) {
+  Element.prototype.matches = Element.prototype.msMatchesSelector ||
+                              Element.prototype.webkitMatchesSelector;
+}
+
 // getAttributeNames polyfill, courtesy of MDN
 if (Element.prototype.getAttributeNames == undefined) {
   Element.prototype.getAttributeNames = function () {
@@ -24,12 +29,14 @@ if (Element.prototype.getAttributeNames == undefined) {
   };
 }
 
-function merge_documents(old_doc, new_doc) {
+function merge_documents(old_doc, new_doc, options) {
+    var options = with_defaults(options);
+
     /*
         First try finding and merging any [elementmerge-whitelist] elements
         This attribute should be specified in NEW document, and element must have an id (matching id of an element in old document)
     */
-    if (merge_whitelist(old_doc, new_doc)) {
+    if (merge_whitelist(old_doc, new_doc, options)) {
         return
     }
 
@@ -40,15 +47,17 @@ function merge_documents(old_doc, new_doc) {
         old_doc.documentElement,
         old_doc.head,
         new_doc.head,
+        options,
     );
     recursive_node_merge(
         old_doc.documentElement,
         old_doc.body,
         new_doc.body,
+        options,
     );
 }
-function merge_whitelist(old_doc, new_doc) {
-    var whitelist = new_doc.querySelectorAll('[elementmerge-whitelist]');
+function merge_whitelist(old_doc, new_doc, options) {
+    var whitelist = new_doc.querySelectorAll(options.whitelist);
     if (!whitelist.length) {
         return false
     }
@@ -61,17 +70,17 @@ function merge_whitelist(old_doc, new_doc) {
             console.error('Unable to find element in old document corresponding to: ',  whitelist[i], '. It either has no id, or no corresponding element in the old document.');
             continue
         }
-        recursive_node_merge(parent, old_element, whitelist[i]);
+        recursive_node_merge(parent, old_element, whitelist[i], options);
     }
     return true
 }
-function merge_from(url) {
+function merge_from(url, options) {
     var r = new XMLHttpRequest();
     r.addEventListener('load', function(event) {
         if (r.status < 200 || r.status > 299) {
             return
         }
-        merge_documents(document, r.response);
+        merge_documents(document, r.response, options);
     });
     r.open('GET', url);
     r.responseType = 'document';
@@ -82,9 +91,14 @@ function merge_from(url) {
     Expose some useful utility functions
 */
 window.elementmerge = {
+    default_options: {
+        whitelist: '',
+        skip: '',
+        nomerge: '',
+    },
     merge_from: merge_from,
-    reload: function() {
-        merge_from(location.href);
+    reload: function(options) {
+        merge_from(location.href, options);
     },
 }
 
@@ -96,15 +110,18 @@ addEventListener('jsformsuccess', function(event) {
     if (!/^text\/html/.test(request.getResponseHeader('content-type'))) return
 
     event.preventDefault();
+
+    // TODO - allow user to specify these selectors as custom attributes on form?
+    var options = {};
     /*
         Note - XMLHTTPRequest has a built-in means of parsing response to a document.
         You have to set responseType to 'document', but you have to set this BEFORE you make the request.
         We can't do that - we want to let the server decide what type of response to return (which may depend on the form values).
     */
-    merge_documents(document, new DOMParser().parseFromString(request.response, 'text/html'));
+    merge_documents(document, new DOMParser().parseFromString(request.response, 'text/html'), options);
 });
 
-function recursive_node_merge(parent, old_node, new_node) {
+function recursive_node_merge(parent, old_node, new_node, options) {
     if (!old_node) {
         parent.appendChild(new_node.cloneNode(true));
         return
@@ -118,8 +135,7 @@ function recursive_node_merge(parent, old_node, new_node) {
         parent.removeChild(old_node);
         return
     }
-
-    if (!should_merge(old_node)) return
+    if (!should_merge(old_node, options)) return
 
     // Now we have two nodes, both of same type
 
@@ -127,14 +143,14 @@ function recursive_node_merge(parent, old_node, new_node) {
     var old_child = old_node.childNodes[0], new_child = new_node.childNodes[0];
     var next_old;
     while (old_child || new_child) {
-        if (should_skip(old_child)) {
+        if (should_skip(old_child, options)) {
             old_child = old_child.nextSibling;
             continue;
         }
 
         // old_child might get deleted, so store reference to next
         next_old = old_child && old_child.nextSibling;
-        recursive_node_merge(old_node, old_child, new_child);
+        recursive_node_merge(old_node, old_child, new_child, options);
         old_child = next_old;
         new_child = new_child && new_child.nextSibling;
     }
@@ -175,22 +191,38 @@ function recursive_node_merge(parent, old_node, new_node) {
         }
     }
 }
-function should_merge(node) {
+function should_merge(node, options) {
     if (!node.hasAttribute) return true // node is not an element
 
     // elements can set this attribute to prevent having their child nodes and attributes merged
     // They will be left alone, and the corresponding node in the new DOM will be ignored
-    return !node.hasAttribute('elementmerge-nomerge')
+    return !node.matches(options.nomerge);
 }
-function should_skip(old_node) {
+function should_skip(old_node, options
+    ) {
     /*
-        Any element that has this attribute will be skipped over.
+        Any element that matches this selector will be skipped over.
 
         You should add this to elements that are added dynamically via javascript (meaning they will be present in old DOM, but not in new DOM).
-
-        TODO - allow users to configure a selector for extra elements to skip (ie. a selector matching select2 elements).
     */
-    return old_node && old_node.hasAttribute && old_node.hasAttribute('elementmerge-skip');
+    return old_node && old_node.matches && old_node.matches(options.skip);
+}
+
+function get_option_selector(options, option_name) {
+    var fixed_value = '[elementmerge-'+option_name+']';
+    var option_value = options[option_name] || elementmerge.default_options[option_name];
+    if (option_value) {
+        return [fixed_value, option_value].join(', ');
+    }
+    return fixed_value;
+}
+function with_defaults(options) {
+    options = options || {};
+    return {
+        whitelist: get_option_selector(options, 'whitelist'),
+        skip: get_option_selector(options, 'skip'),
+        nomerge: get_option_selector(options, 'nomerge'),
+    }
 }
 
 })();
\ No newline at end of file
